*********************************************************
* Program: CODE39.PRG
*
* Author: Jon Walker
* No copyright - Submitted into the public domain
*
* Generate bit pattern for code 39 bar codes
*
* Parameters:
*    msg  = text to decode into a bit pattern
*
* Returns:
*    Bit pattern for code 39 bar code (8 bits high).
*    If an illegal value is found then a null will
*    be returned to the user.
*

parameters m
private ns,ws,nb,wb,wb1,wb2,wb3,wb4
private ws1,ws2,ws3,nb1,nb2,nb3,nb4
private ns1,ns2,ns3,ns4,msg,t,x

  *** if the parameter is not a character then quit
  if type('m')<>"C"
     return ""
  endif
  
  *** Check to see if user added stop/start
  *** character and returns null if true
  if at("*"," "+m)>1
     return ""
  endif
    
  *** add start and stop characters to message
  msg = "*" + upper(m) + "*"

  *** define narrow/wide spaces/bars
  ns = chr(0) + chr(0)
  ws = chr(0) + chr(0) + chr(0) + chr(0)
  nb = chr(255)
  wb = chr(255) + chr(255) + chr(255)

  *** Create the bit patterns we may use       
  WB1 = wb + ns + nb
  WB2 = wb + ns + wb
  WB3 = wb + ws + wb
  WB4 = wb + ws + nb
  
  WS1 = ws + nb + ns
  WS2 = ws + wb + ns
  WS3 = ws + nb + ws
  
  NB1 = nb + ns + wb
  NB2 = nb + ns + nb
  NB3 = nb + ws + wb
  NB4 = nb + ws + nb

  NS1 = ns + wb + ws
  NS2 = ns + nb + ws
  NS3 = ns + nb + ns
  NS4 = ns + wb + ns
  
  *** decode message into bar code bit pattern and place
  *** it into the variable 
  p=""
     
  FOR x = 1 TO len(msg)
     t = substr(msg, x, 1)
       
     do case
        CASE t = "1"
           p = p + WB1 + WS1 + NB1
        CASE t = "2"
           p = p + NB1 + WS1 + NB1
        CASE t = "3"
           p = p + WB2 + WS1 + NB2
        CASE t = "4"
           p = p + NB2 + WS2 + NB1
        CASE t = "5"
           p = p + WB1 + WS2 + NB2
        CASE t = "6"
           p = p + NB1 + WS2 + NB2
        CASE t = "7"
           p = p + NB2 + WS1 + WB2
        CASE t = "8"
           p = p + WB1 + WS1 + WB1
        CASE t = "9"
           p = p + NB1 + WS1 + WB1
        CASE t = "0"
           p = p + NB2 + WS2 + WB1
        CASE t = "A"
           p = p + WB1 + NS2 + NB1
        CASE t = "B"
           p = p + NB1 + NS2 + NB1
        CASE t = "C"
           p = p + WB2 + NS2 + NB2
        CASE t = "D"
           p = p + NB2 + NS1 + NB1
        CASE t = "E"
           p = p + WB1 + NS1 + NB2
        CASE t = "F"
           p = p + NB1 + NS1 + NB2
        CASE t = "G"
           p = p + NB2 + NS2 + WB2
        CASE t = "H"
           p = p + WB1 + NS2 + WB1
        CASE t = "I"
           p = p + NB1 + NS2 + WB1
        CASE t = "J"
           p = p + NB2 + NS1 + WB1
        CASE t = "K"
           p = p + WB1 + NS3 + NB3
        CASE t = "L"
           p = p + NB1 + NS3 + NB3
        CASE t = "M"
           p = p + WB2 + NS3 + NB4
        CASE t = "N"
           p = p + NB2 + NS4 + NB3
        CASE t = "O"
           p = p + WB1 + NS4 + NB4
        CASE t = "P"
           p = p + NB1 + NS4 + NB4
        CASE t = "Q"
           p = p + NB2 + NS3 + WB3
        CASE t = "R"
           p = p + WB1 + NS3 + WB4
        CASE t = "S"
           p = p + NB1 + NS3 + WB4
        CASE t = "T"
           p = p + NB2 + NS4 + WB4
        CASE t = "U"
           p = p + WB4 + NS3 + NB1
        CASE t = "V"
           p = p + NB3 + NS3 + NB1
        CASE t = "W"
           p = p + WB3 + NS3 + NB2
        CASE t = "X"
           p = p + NB4 + NS4 + NB1
        CASE t = "Y"
           p = p + WB4 + NS4 + NB2
        CASE t = "Z"
           p = p + NB3 + NS4 + NB2
        CASE t = "-"
           p = p + NB4 + NS3 + WB2
        CASE t = "."
           p = p + WB4 + NS3 + WB1
        CASE t = " "
           p = p + NB3 + NS3 + WB1
        CASE t = "*"
           p = p + NB4 + NS4 + WB1
        CASE t = "$"
           p = p + NB4 + WS3 + NB2
        CASE t = "/"
           p = p + NB4 + WS1 + NB4
        CASE t = "+"
           p = p + NB4 + NS2 + NB4
        CASE t = "%"
           p = p + NB2 + WS3 + NB4
        otherwise
           *** Invalid code found
           return ""
     endcase
 
     *** Each bar code has a narrow space after it
     p=p+ns
  endfor

  *** Return the bit pattern
  return p

