*******************************************************
* Program: LP-BCODE.PRG
*
* Author: Jon Walker
*
* Barcode interface to print on a many laser printers.
* Create the codes using graphics resolutions of
* 75/100/150 dots per inch.
*
*

set talk off
set status off
set scoreboard off
set escape off


* Get the desired resolution
clear
@  8, 5 say "Bar Code 39 Laser Printer Interface"
@ 10,10 say "Resolution to use"
@ 12,10 say "     1)  75 dpi"
@ 13,10 say "     2) 100 dpi"
@ 14,10 say "     3) 150 dpi"
@ 16,10 say "(ESC quits)"
 

* Wait for a legal key stroke
a=0
do while .not. between(a,1,3) .and. a<>27
   a=inkey(0)-48
enddo


* Was the escape key pressed?
if a=27
   return
endif

* Set up variables for the chosen resolution
baroffset=5-a
dpi=alltrim(substr("   75100150",a*3,3))


* Set the row/column
x=0
y=0

* Main loop of program.  Get label and print it
do while .T.
   clear 
   a=space(15)
   @ 10,10 say "Code39 label to print" get a picture "@!"
   @ 12,10 say "Row to print label   " get y picture "99"
   @ 13,10 say "Column to print label" get x picture "99"
   @ 14,10 say "(ESC quits, PgUp ejects paper)"
   read

   a=trim(a)

   * Was the escape key pressed?
   b=readkey()
   if b=12 .or. b=268
      return
   endif

   * Check printer status, print label if ok
   if printstatus()

      * If the PgUp key was pressed then eject page
      if b=6 .or. b=262
         ??? chr(12)
         loop
      endif

      * Get the bit pattern
      p=code39(trim(a))

      * If an invalid label was entered then loop
      if len(p)=0
         loop
      endif
   
      * Calculate N1 and N2 values for graphics command
      n1 = len(p) % 256
      n2 = int(len(p)/256)

      * Locate print position on the page
      ??? chr(27)+chr(38)+"a"+alltrim(str(y,2))+"R"
      ??? chr(27)+chr(38)+"a"+alltrim(str(x,2))+"C"
      

      * Make the length of p a multiple of 8
      p=p+replicate(chr(0),8-(len(p) % 8))


      * Convert the bar code from a string of verticle
      * bars to a single raster line pattern for the
      * laser printer graphics
      p2=""
      z=chr(0)
      i=1
      do while i<len(p)
         b=0
         for j=1 to 8
            b=iif(substr(p,i,1)=z,b*2,b*2+1)
            i=i+1
         endfor
         p2=p2+chr(b)
      enddo


      * Set the printer resolution and tell it
      * to start accepting graphics
      ??? chr(27)+"*t"+dpi+"R"+chr(27)+"*r1A"

      * Print the raster graphics
      for h = 1 TO 34
         ??? chr(27)+"*b"+alltrim(str(len(p2),3))+"W"+p2
      endfor

      * Get out of graphics mode
      ??? chr(27)+"*rB"

      * Locate print position on the page
      ??? chr(27)+chr(38)+"a"
      ??? alltrim(str(y+baroffset,2))+"R"
      ??? chr(27)+chr(38)+"a"+alltrim(str(x,2))+"C"

      * Print out the text
      ??? space(len(p2)/2.4-len(a)/2)+a

   else

      * Printstatus() failed
      clear
      @ 10,10 say "Printer not responding, press a key."
      clear typeahead
      =inkey(0)
   endif
enddo

return
